#include "libpdf/pdfmisc.h"

#include "libpdf/pdfdictionary.h"
#include "libpdf/pdffont.h"
#include "libpdf/pdfform.h"
#include "libpdf/pdfliteral.h"
#include "libpdf/pdftext.h"

static fieldcount = 0;

PDF_Form PDF_Form_new(PDF_Document doc, PDF_Font font, float size)
{
	PDF_Dictionary retval, fontdict;
	PDF_Queue dastring;
	PDF_Array fields;
	
	/* create the form dictionary and hook it up to the document */
	retval = PDF_Dictionary_new();
	PDF_Object_set_indirect((PDF_Object) retval, PDF_CBOOL_TRUE);
	PDF_Dictionary_add_entry((PDF_Dictionary) doc, "/AcroForm", (PDF_Object) retval);

	/* fill in the required entries in the form dictionary */
	fields = PDF_Array_new();
	PDF_Dictionary_add_entry(retval, "/Fields", (PDF_Object) fields);

	PDF_Dictionary_add_entry(retval, "/DR",
							 PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry((PDF_Dictionary) doc, "/Pages"),
													  "/Resources"));
	dastring = PDF_String_new(NULL);
	PDF_Text_set_font(dastring, font, size);
	PDF_Dictionary_add_entry(retval, "/DA", (PDF_Object) dastring);

	return retval;
}
PDF_Field PDF_FormFieldText_new(PDF_Form form, PDF_Page page, float x, float y, float wid, float ht)
{
	PDF_Stream appearance;
	PDF_Dictionary field, annot, ap;
	PDF_Array rect, bbox;
	char namebuf[10];

	/* Create the field and connect to form */
	field = PDF_Dictionary_new();
	PDF_Object_set_indirect((PDF_Object) field, PDF_CBOOL_TRUE);
	PDF_Queue_append((PDF_Queue) PDF_Dictionary_get_entry(form, "/Fields"), (PDF_Object) field);
	PDF_Dictionary_add_entry(field, "/Kids", (PDF_Object) PDF_Array_new());

	/* Field entries */
	/* The field title "T" is supposed to be optional, but I don't get
	   interaction without it.  Even better, the fields seem to need
	   to have different names */
	snprintf(namebuf, 10, "Field%3d", fieldcount++);
	PDF_Dictionary_add_entry(field, "/T", (PDF_Object) PDF_String_new(namebuf));
	PDF_Dictionary_add_entry(field, "/FT", (PDF_Object) PDF_Literal_new("/Tx"));

	/* create the annotation and connect to field and page */
	annot = PDF_Dictionary_new();
	PDF_Object_set_indirect((PDF_Object) annot, PDF_CBOOL_TRUE);
	PDF_Dictionary_add_entry(annot, "/Parent", (PDF_Object) field);
	PDF_Dictionary_add_entry(annot, "/P", (PDF_Object) page);
	PDF_Queue_append((PDF_Queue) PDF_Dictionary_get_entry(field, "/Kids"), (PDF_Object) annot);
	PDF_Queue_append((PDF_Queue) PDF_Dictionary_get_entry(page, "/Annots"), (PDF_Object) annot);

	/* Annotation fields */
	PDF_Dictionary_add_entry(annot, "/Type", (PDF_Object) PDF_Literal_new("/Annot"));
	PDF_Dictionary_add_entry(annot, "/Subtype", (PDF_Object) PDF_Literal_new("/Widget"));
	rect = PDF_Array_new();
	PDF_Queue_append(rect, (PDF_Object) PDF_Float_new(x));
	PDF_Queue_append(rect, (PDF_Object) PDF_Float_new(y));
	PDF_Queue_append(rect, (PDF_Object) PDF_Float_new(x+wid));
	PDF_Queue_append(rect, (PDF_Object) PDF_Float_new(y+ht));
	PDF_Dictionary_add_entry(annot, "/Rect", (PDF_Object) rect);
	PDF_Dictionary_add_entry(annot, "/F", (PDF_Object) PDF_Int_new(4));
	
	/* create the /AP dictionary and hook up to the annotation */
	ap = PDF_Dictionary_new();
	PDF_Dictionary_add_entry(annot, "/AP", (PDF_Object) ap);

	/* create the appearance stream and connect it to the /AP entry */
	appearance = PDF_Stream_new();
	PDF_Object_set_indirect((PDF_Object) appearance, PDF_CBOOL_TRUE);
	PDF_Dictionary_add_entry(ap, "/N", (PDF_Object) appearance);

	/* appearance stream fields */
	PDF_Stream_set_param(appearance, "/Subtype", (PDF_Object) PDF_Literal_new("/Form"));
	
	bbox = PDF_Array_new();
	PDF_Queue_append(bbox, (PDF_Object) PDF_Float_new(0));
	PDF_Queue_append(bbox, (PDF_Object) PDF_Float_new(0));
	PDF_Queue_append(bbox, (PDF_Object) PDF_Float_new(wid));
	PDF_Queue_append(bbox, (PDF_Object) PDF_Float_new(ht));
	PDF_Stream_set_param(appearance, "/BBox", (PDF_Object) bbox);
	PDF_Stream_set_param(appearance, "/Resources", PDF_Dictionary_get_entry(form, "/DR"));
	
	PDF_Stream_append(appearance, (PDF_Object) PDF_Literal_new("/Tx BMC EMC"));
	
	return (PDF_Field) field;
}
