#include "libpdf/pdfarray.h"
#include "libpdf/pdfdictionary.h"
#include "libpdf/pdfdocument.h"
#include "libpdf/pdfgraphics.h"
#include "libpdf/pdfliteral.h"
#include "libpdf/pdfpage.h"

PDF_Page PDF_Page_new(PDF_Document doc)
{
	PDF_Dictionary retval;
	PDF_Array annots;
	PDF_Array contents;
	
	retval = PDF_Dictionary_new();
	if (retval == NULL)
		return NULL;
	
	if (PDF_Dictionary_add_entry(retval, "/Type", (PDF_Object) PDF_Literal_new("/Page")) == PDF_CBOOL_FALSE) {
		free(retval);
		return NULL;
	}

	contents = PDF_Array_new();
	if (PDF_Dictionary_add_entry(retval, "/Contents", (PDF_Object) contents) == PDF_CBOOL_FALSE) {
		free(retval);
		return NULL;
	}

	PDF_Dictionary_add_entry(retval, "/Resources", PDF_Dictionary_get_entry((PDF_Dictionary) PDF_Dictionary_get_entry(doc, "/Pages"), "/Resources"));

	annots = PDF_Array_new();
	if (PDF_Dictionary_add_entry(retval, "/Annots", (PDF_Object) annots) == PDF_CBOOL_FALSE) {
		free(retval);
		return NULL;
	}																		

	PDF_Object_set_indirect((PDF_Object) retval, PDF_CBOOL_TRUE);

	PDF_Document_add_Page(doc, retval);
	
	return retval;
}

PDF_Graphics PDF_PageContent_new(PDF_Page page)
{
	PDF_Stream newstream;

	newstream = PDF_Stream_new();
	PDF_Queue_append((PDF_Queue) PDF_Dictionary_get_entry((PDF_Dictionary) page, "/Contents"), (PDF_Object) newstream);

	return (PDF_Graphics) PDF_Stream_get_content(newstream);
}
