#ifndef GRAPHICS_H
#define GRAPHICS_H

#include "libpdf/pdfqueue.h"

typedef PDF_Queue PDF_Graphics;

#include "libpdf/pdfarray.h"
#include "libpdf/pdfimage.h"
#include "libpdf/pdfstream.h"

typedef enum {PDF_CAPSTYLE_BUTT=0, PDF_CAPSTYLE_ROUND=1, PDF_CAPSTYLE_PROJECTING=2} PDF_CapStyle;
typedef enum {PDF_JOINSTYLE_MITER=0, PDF_JOINSTYLE_ROUND=1, PDF_JOINSTYLE_BEVEL=2} PDF_JoinStyle;
/* this enum MUST MATCH the PDF_Intent_Strings array in graphics.c */
typedef enum {PDF_COLORINTENT_ABSOLUTECOLORIMETRIC=0, PDF_COLORINTENT_RELATIVECOLORIMETRIC=1,
			  PDF_COLORINTENT_SATURATION=2, PDF_COLORINTENT_PERCEPTUAL=3} PDF_ColorIntent;
			  
PDF_Graphics PDF_Graphics_new(PDF_Graphics contentstream);
PDF_CBool PDF_Graphics_append(PDF_Graphics graphics, PDF_Object obj);
PDF_CBool PDF_Graphics_append_op(PDF_Graphics graphics, PDF_CString format, ...);

/* graphics state operators */
PDF_CBool PDF_Graphics_concat_matrix(PDF_Graphics obj, float a, float b, float c, float d, float e, float f);
PDF_CBool PDF_Graphics_translate(PDF_Graphics obj, float x, float y);
PDF_CBool PDF_Graphics_rotate(PDF_Graphics obj, float theta);
PDF_CBool PDF_Graphics_scale(PDF_Graphics obj, float x, float y);

PDF_CBool PDF_Graphics_line_width(PDF_Graphics obj, float width);
PDF_CBool PDF_Graphics_line_cap(PDF_Graphics obj, PDF_CapStyle style);
PDF_CBool PDF_Graphics_line_join(PDF_Graphics obj, PDF_JoinStyle style);
PDF_CBool PDF_Graphics_mitre_limit(PDF_Graphics obj, float limit);
PDF_CBool PDF_Graphics_dash(PDF_Graphics stream, float dashArray[], float phase);
PDF_CBool PDF_Graphics_intent(PDF_Graphics obj, PDF_ColorIntent intent);
PDF_CBool PDF_Graphics_flatness(PDF_Graphics stream, float flatness);
PDF_CBool PDF_Graphics_image(PDF_Graphics gs, PDF_Image image);

/* see comment in graphics.c
 PDF_CBool PDF_Graphics_dictionary(PDF_Stream stream, PDF_CString dictName);*/

#endif
