#ifndef OBJECT_H
#define OBJECT_H

#include <stdio.h>

typedef struct PDF_object *PDF_Object;

#include "libpdf/pdfmisc.h"
#include "libpdf/pdfqueue.h"

#include "libpdf/pdfheader.h"
#include "libpdf/pdfxref.h"

struct PDF_object {
	struct PDF_header header;
};

/* note that there is no general PDF_Object_new since the
   initializations requred by the various types call for too much
   variation in the parameters (if there is ever a real need for this, it
   can be done with varargs, of course.

   PDF_Object_create doesn't do the same thing PDF_Object_new would, and
   is in some sense exactly the opposite of PDF_Object_delete and
   PDF_Object_write.  PDF_Object_create is called by the various object
   constructructors; it allocates and sets up a an object header, and
   allocates space for the object itself but does not initialize that
   space. */
PDF_Object PDF_Object_create(int size, PDF_ObjType type, PDF_CBool indirect);

/* Set/get an arbitrary PDF object's indirect flag */
void PDF_Object_set_indirect(PDF_Object obj, PDF_CBool flag);
PDF_CBool PDF_Object_get_indirect(PDF_Object obj);

/* Delete an arbitrary PDF object.  It calls the custom destructor
   for the object, frees the header, and frees the object itself (notice
   it is expected that the custom object destructor does not free its own
   space */
void PDF_Object_delete(PDF_Object obj);

/* Write an arbitrary PDF object to the output buffer, using the
   representation described in the PDF ref manual section 3.2.  */
int PDF_Object_write(PDF_Object obj, FILE *ostream, int index,
					 PDF_CBool put_indirect, PDF_Xref_Table xref,
					 PDF_Queue indqueue);

#endif
