#include <malloc.h>
#include <stdarg.h>
#include <string.h>

#include "libpdf/pdfliteral.h"
#include "libpdf/pdfobject.h"
#include "libpdf/pdfqueue.h"

PDF_Queue PDF_Queue_new(PDF_CString before, PDF_CString between, PDF_CString after)
{
	PDF_Queue retval;

	if (((retval = (PDF_Queue) PDF_Object_create(sizeof(struct PDF_queue), PDF_QUEUE, PDF_CBOOL_FALSE)) == NULL) ||
		((retval->before = strdup(before)) == NULL) ||
		((retval->between = strdup(between)) == NULL) ||
		((retval->after = strdup(after)) == NULL)) {
		
		if (retval->before != NULL)
			free(retval->before);
		if (retval->between != NULL)
			free(retval->between);
		if (retval->after != NULL)
			free(retval->after);
		if (retval != NULL)
			free(retval);
	}
	
	retval->first = NULL;
	retval->last = NULL;

	return retval;
}

PDF_Queue_Node PDF_Queue_Node_new(PDF_Object obj)
{
	PDF_Queue_Node retval;

	retval = malloc(sizeof(struct PDF_queue_node));
	if (retval == NULL)
		return NULL;

	retval->data = obj;
	retval->next = NULL;

	return  retval;
}

PDF_Queue_Node PDF_Queue_Node_delete(PDF_Queue_Node obj)
{
	PDF_Queue_Node retval;

	retval = obj->next;
	PDF_Object_delete(obj->data);
	free(obj);

	return retval;
}

void PDF_Queue_delete(PDF_Queue queue)
{
	PDF_Queue_Node elem;

	for (elem = PDF_Queue_get_first(queue); elem != NULL; elem = PDF_Queue_Node_delete(elem)) /* empty */ ;
}

PDF_CString PDF_Queue_get_before(PDF_Queue queue)
{
	return queue->before;
}

PDF_CString PDF_Queue_get_between(PDF_Queue queue)
{
	return queue->between;
}

PDF_CString PDF_Queue_get_after(PDF_Queue queue)
{
	return queue->after;
}

int PDF_Queue_write(PDF_Queue queue, FILE *ostream, int index,
					PDF_CBool put_indirect, PDF_Xref_Table xref,
					PDF_Queue indqueue)
{
	PDF_Queue_Node entry;

	int numwritten;

	numwritten = fprintf(ostream, "%s", PDF_Queue_get_before(queue));
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	for (entry = PDF_Queue_get_first(queue); entry != NULL; entry = PDF_Queue_Node_get_next(entry)) {
		index = PDF_Object_write(PDF_Queue_Node_get_data(entry),
								 ostream, index, put_indirect, xref, indqueue);
		if (index < 0)
			return -1;
		
		if (PDF_Queue_Node_get_next(entry) != NULL) {
			numwritten = fprintf(ostream, "%s", PDF_Queue_get_between(queue));
			if (numwritten < 0)
				return -1;
			index += numwritten;
		}
	}

	numwritten = fprintf(ostream, "%s", PDF_Queue_get_after(queue));
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	return index;
}

PDF_Queue_Node PDF_Queue_get_first(PDF_Queue queue)
{
	return queue->first;
}

PDF_Queue_Node PDF_Queue_get_last(PDF_Queue queue)
{
	return queue->last;
}

PDF_CBool PDF_Queue_append(PDF_Queue queue, PDF_Object newobj)
{
	PDF_Queue_Node newnode;

	newnode = malloc(sizeof(struct PDF_queue_node));
	if (newnode == NULL)
		return PDF_CBOOL_FALSE;

	newnode->data = newobj;
	newnode->next = NULL;

	if (queue->last != NULL)
		queue->last->next = newnode;
	else
		queue->first = newnode;

	queue->last = newnode;

	return PDF_CBOOL_TRUE;
}

PDF_CBool PDF_Queue_append_op(PDF_Queue queue, PDF_CString format, ...)
{
	char contentbuf[PDF_SMALLBUFSIZ];
	va_list arg_list;
	
	va_start (arg_list, format);
	vsnprintf(contentbuf, PDF_SMALLBUFSIZ, format, arg_list);
	va_end(arg_list);

	return PDF_Queue_append(queue, (PDF_Object) PDF_Literal_new(contentbuf));
}

PDF_Object PDF_Queue_Node_get_data(PDF_Queue_Node node)
{
	return node->data;
}

PDF_Queue_Node PDF_Queue_Node_get_next(PDF_Queue_Node node)
{
	return node->next;
}
