#include <malloc.h>
#include <stdio.h>
#include <string.h>

#include "libpdf/pdfmisc.h"
#include "libpdf/pdfheader.h"
#include "libpdf/pdfobject.h"
#include "libpdf/pdfdictionary.h"
#include "libpdf/pdfint.h"
#include "libpdf/pdfstream.h"

PDF_Stream PDF_Stream_new()
{
	PDF_Stream retval;

	retval = (PDF_Stream) PDF_Object_create(sizeof(struct PDF_stream), PDF_STREAM, PDF_CBOOL_TRUE);
	if (retval == NULL)
		return NULL;

	retval->dictionary = PDF_Dictionary_new();
	if (retval->dictionary == NULL) {
		free(retval);
		
		return NULL;
	}

	retval->content = PDF_Queue_new("", "", "");
	if (retval->content == NULL) {
		free(retval->dictionary);
		free(retval);

		return NULL;
	}
	
	return retval;
}

void PDF_Stream_delete(PDF_Stream obj)
{
	PDF_Object_delete((PDF_Object) PDF_Stream_get_dictionary(obj));
	PDF_Object_delete((PDF_Object) PDF_Stream_get_content(obj));
}

PDF_CBool PDF_Stream_set_param(PDF_Stream stream, PDF_CString paramname, PDF_Object paramval)
{
	return PDF_Dictionary_add_entry(PDF_Stream_get_dictionary(stream), paramname, paramval);
}

PDF_CBool PDF_Stream_append(PDF_Stream obj, PDF_Object newobj)
{
	return PDF_Queue_append(PDF_Stream_get_content(obj), newobj);
}

PDF_Dictionary PDF_Stream_get_dictionary(PDF_Stream obj)
{
	return obj->dictionary;
}

PDF_Queue PDF_Stream_get_content(PDF_Stream obj)
{
	return obj->content;
}
/* I'll take the stream and put it into a temp buffer (hope this is
   long enough...  this'll take some rethought....) and then compress
   it into the final buffer.  The clever idea of using an indirect
   object for the stream length so I can put it out before the stream
   content is taken from ghostscript's pdf generator.
*/
int PDF_Stream_write(PDF_Stream stream, FILE *ostream, int index,
					 PDF_CBool put_indirect, PDF_Xref_Table xref,
					 PDF_Queue indqueue)
{
	/*	char buffer[1000000];*/

	int index2;
	int numwritten;
	int result;
	int i, objlen;
	long destlen;
	PDF_Int lengthobj;

	lengthobj = PDF_Int_new(0);
	PDF_Object_set_indirect((PDF_Object) lengthobj, PDF_CBOOL_TRUE);

	PDF_Stream_set_param(stream, "/Length", (PDF_Object) lengthobj);
	/*PDF_Stream_set_param(stream, "/Filter", PDF_Literal_new("/FlateDecode"));*/
	
	index = PDF_Object_write((PDF_Object) PDF_Stream_get_dictionary(stream),
							 ostream, index, PDF_CBOOL_FALSE, xref, indqueue);
	if (index < 0)
		return -1;
	
	numwritten = fprintf(ostream, "stream\n");
	if (numwritten < 0)
		return -1;
	index += numwritten;

	/* the following code was put in back when I was trying to do
	   output file compression, and was doing output by building an
	   in-memory buffer and then dumping the whole thing.  Doing
	   direct output to an output stream has made compressing streams
	   more trouble than it's worth on this pass.

	   It's actually not clear to me that compressing the streams is
	   ever worthwhile.  You get better compression if you just
	   generate the whole PDF file and compress the result anyway.
	*/

	/*	objlen = PDF_Object_put(PDF_Stream_get_content(stream), TRUE, buffer, 0, 1000000, xref, indqueue);
	destlen = strlen-index;
	result = compress(outstr+index, &destlen, buffer, objlen);
	objlen = destlen;

	if (objlen > strlen - index)
		return -1;

	*/
	index2 = PDF_Object_write((PDF_Object) PDF_Stream_get_content(stream),
							  ostream, index, PDF_CBOOL_FALSE, xref, indqueue);
	objlen = index2 - index;
	index = index2;

	numwritten = fprintf(ostream, "\nendstream\n");
	if (numwritten < 0)
		return -1;
	index += numwritten;

	PDF_Int_set_value(lengthobj, objlen);
	
	return index;
}
