#include <malloc.h>

#include "libpdf/pdfxref.h"

PDF_Xref_Table PDF_Xref_Table_new()
{
	PDF_Xref_Table retval;

	retval = malloc(sizeof(struct PDF_xref_table));
	if (retval == NULL)
		return NULL;

	retval->maxref = -1;
	retval->maxused = 0;
	retval->entries = NULL;

	return retval;
}

PDF_CBool PDF_Xref_Table_append(PDF_Xref_Table table, int objnum, int offset, int version)
{
	PDF_Xref newtable;
	int entrynum;
	
	/* if the entry buffer isn't long enough, reallocate it.  We'll
	   reallocate the table to be twice as large as the entry number
	   that overflowed it */
	if (objnum > table->maxref) {
		newtable = realloc(table->entries, sizeof(struct PDF_xref) * (objnum << 1));
		if (newtable == NULL)
			return(PDF_CBOOL_FALSE);
		
		for (entrynum = table->maxref+1; entrynum < objnum << 1; entrynum++) {
			newtable[entrynum].offset = 0;
			newtable[entrynum].version = 0;
		}
		
		table->maxref = (objnum << 1) - 1;
		table->entries = newtable;
	}

	table->entries[objnum].offset = offset;
	table->entries[objnum].version = version;

	if (objnum > table->maxused)
		table->maxused = objnum;

	return(PDF_CBOOL_TRUE);
}

int PDF_Xref_Table_write(PDF_Xref_Table table, FILE *ostream, int index,
						 PDF_CBool put_indirect, PDF_Xref_Table xref,
						 PDF_Queue indqueue)
{
	int i, numwritten;
	
	numwritten = fprintf(ostream, "xref\n0 %d\n", table->maxused+1);
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	numwritten = fprintf(ostream, "0000000000 65535 f \n");
	if (numwritten < 0)
		return -1;
	index += numwritten;
	
	for (i = 1; i <= table->maxused; i++) {
		numwritten = fprintf(ostream, "%010d %05d n \n",
							 table->entries[i].offset,
							 table->entries[i].version);
		if (numwritten < 0)
			return -1;
		index += numwritten;
	}

	return index;
}

int PDF_Xref_Table_new_objnum(PDF_Xref_Table obj)
{
	obj->maxused++;
	
	return obj->maxused;
}
